<?php
// Exit if accessed directly
if (!defined('ABSPATH')) exit;


#===============================================
# 	Configure Mollie Gateway for Events Manager Pro.
#===============================================
Class EM_Gateway_Mollie extends EM_Gateway {

	var $gateway = 'mollie';
	var $title = 'Mollie';
	var $status = 4;
	var $status_txt = 'Awaiting Mollie Payment';
	var $button_enabled = false;
	var $payment_return = true;
	var $supports_multiple_bookings = true;

	var $transaction_detail = array(
		'Mollie Dashboard',
		'https://www.mollie.com/dashboard/payments/%s',
		'https://www.mollie.com/dashboard/payments/%s'
		);


	#===============================================
	public function __construct() {
		parent::__construct();
		global $EM_Mollie;
		$mollie = $EM_Mollie->start_mollie();

		if( is_object($mollie) ) {
			$this->mollie = $mollie;
		}

		// Check if the gateway is activated (= toggled).
		if (parent::is_active()) {
			add_action('em_gateway_js', array($this, 'em_gateway_js'));
			add_filter('em_booking_validate', array($this, 'em_booking_validate'), 2, 2);
		}
		add_filter('the_content', array($this, 'handle_mollie_customer_return'));
	}


	#===============================================
	# 	Add JavaScript to prevent WordPress error "Header already sent in..."
	#===============================================
	function em_gateway_js() {
		include dirname( __FILE__ ) . '/gateway.mollie.js';
	}


	#===============================================
	# 	Booking Interception - functions that modify booking object behaviour
	#===============================================
	function booking_form() {
		global $EM_Booking, $EM_Event, $EM_Ticket, $EM_Mollie;

		echo get_option('em_mollie_form');

		// Show Active Payment Methods?
		if( get_option('em_mollie_show_methods') == "yes" && is_object($EM_Mollie->start_mollie()) ) {
			$mollie 	= $EM_Mollie->start_mollie();
			$methods 	= $mollie->methods->all();

			echo '<div class="mollie-methods" id="mollie-methods">';
			foreach( $methods as $method ) {
				echo sprintf( '<img src="%s" srcset="%s" alt="%s" title="%s" width="50px" height="38px"> ',
					htmlspecialchars($method->image->size1x),
					htmlspecialchars($method->image->size2x),
					$EM_Mollie->translate( htmlspecialchars($method->id) ),
					$EM_Mollie->translate( htmlspecialchars($method->id) )
				);
			}
			echo '</div>';
		}
		return;
	}


	#===============================================
	# 	Hook into booking validation and check validate payment type if present.
	#===============================================
	function em_booking_validate($result, $EM_Booking) {
		if (isset( $_POST['paymentType'] ) && empty( $_POST['paymentType'] )) {
			$EM_Booking->add_error('Please select a payment method.');
			$result = false;
		}

		$api_key = get_option('em_mollie_api_key');
		if( !isset($api_key) || empty($api_key) ) {
			$EM_Booking->add_error('Mollie API Key is not found.');
			$result = false;
		}

		return $result;
	}


	#===============================================
	# 	After form submission by user, add Mollie vars and show feedback message.
	#===============================================
	function booking_form_feedback( $return, $EM_Booking = true ) {
		if( !empty($return['errors']) ) {
			return $return;
		}

		global $wpdb, $wp_rewrite, $EM_Notices, $EM_Booking, $EM_Event, $EM_Mollie;
		$booking_id  		= $EM_Booking->booking_id;
		$timestamp   		= time();
		$wildcard_default 	= __('%spaces% tickets for %blog_name%', 'stonehenge-em-mollie');
		$wildcards 			= !empty( get_option('em_mollie_description')) ? get_option('em_mollie_description') : $wildcard_default;

		// If in Multiple Bookings Mode and forbidden wildcards are still used, set $description back to default.
		if( get_option('dbem_multiple_bookings') && strpos( $wildcards, '%event_' ) ) {
			$wildcards = $wildcard_default;
		}

		// Double check $EM_Booking is an EM_Booking object and that we have a booking awaiting payment.
		if (is_object($EM_Booking) && $this->uses_gateway($EM_Booking) ) {
			if (!empty($return['result']) && $EM_Booking->get_price() > 0 && $EM_Booking->booking_status == $this->status ) {

				$description = apply_filters('em_mollie_wildcards', $wildcards);
				$mollie 	= $EM_Mollie->start_mollie();
				$request 	= $mollie->payments->create( array(
					"amount"  		=> array(
						"currency" 		=> get_option('dbem_bookings_currency'),
						"value"   		=> number_format( $EM_Booking->get_price(), 2)
					),
					"description" 	=> $description,
					"redirectUrl" 	=> $this->get_mollie_return_url() . "?em_mollie_return={$EM_Booking->booking_id}",
					"webhookUrl" 	=> $this->get_payment_return_url(),
					"locale"   		=> get_locale(),									// Set checkout page to blog's locale.
					"sequenceType"  => "oneoff",  										// Default for single payment.
					"metadata"  	=> array(
						"booking_id" 	=> $EM_Booking->booking_id,
						"name"    		=> $EM_Booking->get_person()->get_name(),
						"email"   		=> $EM_Booking->get_person()->user_email,
					),
				));

				$return['message']  = get_option('em_mollie_message_redirect');
				$mollie_return  	= array(
					'mollie_url'   		=> $request->getCheckoutUrl(),
					'mollie_vars'   	=> $request
				);
				$return = array_merge( $return, $mollie_return );

			} else {
				// Return a free message and do not redirect to Mollie.
				$return['message'] 	= get_option('em_mollie_message_free');
				$mollie_return 		= array(
					'mollie_url' 		=> $this->get_mollie_return_url() ."?em_mollie_free={$EM_Booking->booking_id}"
				);
				$return = array_merge( $return, $mollie_return );
			}
		}
		return $return;
	}


	#===============================================
	# 	Determine the redirect url after Mollie payment.
	#===============================================
	function get_mollie_return_url() {
		global $EM_Event;

		 if( get_option('em_mollie_return_page') ){
			 $redirect_url = get_permalink(get_option( 'em_mollie_return_page') );
		 }
		 else {
			 if( get_option('dbem_multiple_bookings') )	{
				 $redirect_url = get_permalink( get_option( 'dbem_events_page' ) );
			 }
			 else {
				 $redirect_url = $EM_Event->output("#_EVENTURL");
			 }
		 }
		return $redirect_url;
	}


	#===============================================
	# 	Handle content when a user returns from Mollie after payment.
	#===============================================
	function handle_mollie_customer_return( $content ) {
		global $EM_Mollie;
	 	if( strpos($_SERVER['REQUEST_URI'], 'em_mollie_free') ) {
			echo sprintf( '<p><div class="em-booking-message em-booking-message-success">%s</div></p>', get_option('em_mollie_message_free'));
			return $content;
		}
		elseif( strpos($_SERVER['REQUEST_URI'], 'em_mollie_return') ) {
			global $wpdb;
			$booking_id = $_REQUEST['em_mollie_return'];
			$status  	= $wpdb->get_var( "SELECT `transaction_status` FROM ". EM_TRANSACTIONS_TABLE ." WHERE `booking_id` = {$booking_id}");
			if( empty($status) ) {
				return $content;
			}
			else {
				$class = $feedback = $result = '';
				switch( $status ) {
					case 'paid':
						$class 		= 'success';
						$feedback 	= get_option( 'dbem_booking_feedback' );
						break;
					case 'canceled':	// IMPORTANT: Mollie uses US spelling!
					case 'cancelled':
					case 'expired':
					case 'failed' :
						$class 		= 'error';
						$feedback 	= get_option( 'dbem_booking_feedback_error' );
						break;
					case 'open':
					case 'pending':
					default:
						$class 		= 'info';
						$feedback 	= get_option( 'dbem_booking_feedback_pending' );
						// Add styling fot this status only - use EM css for the others.
						$result    .= '<style>.em-booking-message-info { background-color: #d1ecf1; border: 1px solid #0c5460;}</style>';
						break;
				}
				$feedback_status = sprintf( '%s: <strong>%s</strong><br />',
					__('The status of your payment is', 'stonehenge-em-mollie'),
					strtoupper( $EM_Mollie->translate($status))
					);
				$show_status 	= (get_option('em_mollie_show_status') != 'yes') ? '' : $feedback_status;
				$show_feedback 	= (get_option('em_mollie_show_feedback') != 'yes') ? '' : $feedback;
				$result 		.= sprintf( '<div class="em-booking-message em-booking-message-%s">%s%s</div>', $class, $show_status, $show_feedback );
				$result 		.= '<p><a href="'. get_permalink( get_option('dbem_events_page') ) .'"><input type="button" value="'. __('Continue', 'stonehenge-em-mollie') .'" class="button mollie-transaction"></a></p>';
				return $result;
			}
		}
		else {
			return $content;
		}
	}


	#===============================================
	# 	When Mollie calls the webhook, update database, update Booking Status & send emails.
	#===============================================
	function handle_payment_return() {
		// Catch Mollie's transaction ID, sent as "id=tr_xxxxxxxxxx".
		$post   	= file_get_contents('php://input');
		$post_id 	= explode("=", $post);
		$mollie_id  = $post_id[1];


		// Fetch all transaction info from Mollie.
		global $EM_Mollie;
		if( !is_object($EM_Mollie->start_mollie()) ) {
			return;
		}

		$mollie 	= $EM_Mollie->start_mollie();
		$payment 	= $mollie->payments->get($mollie_id);
		$timestamp  = date('Y-m-d H:i:s', strtotime($payment->createdAt));
		$booking_id = $payment->metadata->booking_id;
		$EM_Booking = em_get_booking($booking_id);

		if (!empty( $EM_Booking->booking_id )) {
			$EM_Booking->manage_override = true;
			$user_id 	= $EM_Booking->person_id;
			if ($payment->isPaid() && !$payment->hasRefunds() && !$payment->hasChargebacks()) {
				$this->record_transaction( $EM_Booking, $payment->amount->value, $payment->amount->currency, $timestamp, $mollie_id, $payment->status, '' );
				$EM_Booking->approve(true, true);
				do_action('em_payment_processed', $EM_Booking, $this);
			}
			elseif ($payment->isOpen() || $payment->isPending()) {
				$EM_Booking->set_status(4);
				do_action('em_payment_processed', $EM_Booking, $this);
			}
			elseif ($payment->isCanceled() || $payment->isFailed() || $payment->isExpired()) {
				// Mollie uses US spelling.
				$payment->status = ($payment->status != 'canceled') ? $payment->status : 'cancelled';
				$this->record_transaction( $EM_Booking, $payment->amount->value, $payment->amount->currency, $timestamp, $mollie_id, $payment->status, '' );
				if( get_option('em_mollie_send_cancel_mail') != 'yes' ) {
					$EM_Booking->set_status(3, false);
				}
				else {
					$EM_Booking->set_status(3);
				}

				do_action('em_payment_processed', $EM_Booking, $this);
			}
			elseif ($payment->hasChargebacks()) {
				$note = "Payment has been charged back.";
				$this->record_transaction($EM_Booking, $payment->amount->value, $payment->amount->currency, $timestamp, $mollie_id, 'charged back', $note);
				$EM_Booking->set_status(3);
				do_action('em_payment_processed', $EM_Booking, $this);
			}
			elseif ($payment->hasRefunds()) {
				// Fetch detailed info for refund from Mollie.
				foreach( $payment->refunds() as $refund ) {
					$date 		= $EM_Mollie->get_localized_time($refund->createdAt);
					$note 		= "(Partially) Refunded on {$date}";
					$this->record_transaction($EM_Booking, $payment->amount->value, $payment->amount->currency, $timestamp, $mollie_id, 'refunded', $note);
				}
				do_action('em_payment_processed', $EM_Booking, $this);
			}
		}
		return;
	}


	#===============================================
	# 	Gateway Settings Functions
	#===============================================
	function define_settings_fields() {
		$dashboard_url = 'https://www.mollie.com/dashboard/developers/api-keys';
		$wildcard_default = __('%spaces% tickets for %blog_name%', 'stonehenge-em-mollie');

		$fields_array = array(
			array(
				'id' 		=> 'api_key',
				'label' 	=> __('Molie API Key', 'stonehenge-em-mollie'),
				'type' 		=> 'text',
				'default' 	=> '',
				'help' 		=> sprintf(
					__('Obtain your Live or Test API Key from your <a href=%s target="_blank">Mollie Dashboard</a>.', 'stonehenge-em-mollie'),
					$dashboard_url
				),
			),
			array(
				'id'		=> 'show_methods',
				'label'  	=> __('Display Payment Methods', 'stonehenge-em-mollie'),
				'type' 		=> 'select',
				'default' 	=> 'yes',
				'help' 		=> __('Display small images of the activated payment methods on your booking form?', 'stonehenge-em-mollie') .'<br>'. sprintf( __( 'You can activate/deactivate each payment method individually in your <a href=%s target="_blank">Mollie Dashboard</a>.', 'stonehenge-em-mollie'), $dashboard_url ),
			),
			array(
				'id' 		=> 'message_free',
				'label' 	=> __('Free Booking Message', 'stonehenge-em-mollie'),
				'type' 		=> 'text',
				'default' 	=> __('Thank you for your booking.<br>You will receive a confirmation email soon.', 'stonehenge-em-mollie'),
				'help' 		=> __('This message will be shown if the total booking price = 0.00. Your customer will <u>not</u> be redirected to Mollie.', 'stonehenge-em-mollie'),
			),
			array(
				'id' 		=> 'message_redirect',
				'label' 	=> __('Redirect Message', 'stonehenge-em-mollie'),
				'type' 		=> 'text',
				'default' 	=> __('Redirecting to complete your online payment...', 'stonehenge-em-mollie'),
				'help' 		=> __('This message will be shown right before your customer is redirected to Mollie.', 'stonehenge-em-mollie'),
			),
			array(
				'id'		=> 'return_page',
				'label' 	=> __('Return Page', 'stonehenge-em-mollie'),
				'type'  	=> 'page',
				'default'  	=> '',
				'help'   	=> __('Your customer will be redirected back to this page after the payment.', 'stonehenge-em-mollie').'<br>'. __('Leave blank to use the Single Event Page (in Single Bookings Mode) or the Events Page (in Multiple Bookings Mode).', 'stonehenge-em-mollie'),
			),
			array(
				'id' 		=> 'show_status',
				'label' 	=> __('Display Payment Status', 'stonehenge-em-mollie'),
				'type' 		=> 'select',
				'default' 	=> 'yes',
				'help' 		=> __('Display the payment status on the Return Page?', 'stonehenge-em-mollie') .'<br><code>'. __('The status of your payment is:', 'stonehenge-em-mollie') .' [status]</code>',
			),
			array(
				'id' 		=> 'show_feedback',
				'label' 	=> __('Display Feedback Messages', 'stonehenge-em-mollie'),
				'type' 		=> 'select',
				'default' 	=> 'yes',
				'help' 		=> sprintf( __('Display the booking feedback messages as set in your <a href=%s target="_blank">Events Manager Settings</a>?', 'stonehenge-em-mollie'), admin_url('edit.php?post_type=event&page=events-manager-options#bookings') ),
			),
			array(
				'id' 		=> 'description',
				'label' 	=> __('Payment Description', 'stonehenge-em-mollie'),
				'type' 		=> 'text',
				'default' 	=> __( $wildcard_default, 'stonehenge-em-mollie' ),
				'help'      => sprintf( '%s = %s<br><br><strong>%s</strong><code style="display:table-cell; width:auto; padding:8px;">%s<br>%s<br>%s<br>%s</code><br><strong>%s</strong><code style="display:table-cell; width:auto; padding:8px;">%s<br>%s</code>',
					__('Default', 'stonehenge-em-mollie'),
					__( $wildcard_default, 'stonehenge-em-mollie' ),
					__('The following wildcards can be used to create the payment description:', 'stonehenge-em-mollie'),
					'%blog_name% &rArr; ' . __('The name of your website', 'stonehenge-em-mollie'),
					'%booking_id% &rArr; ' . __('The ID of the current booking', 'stonehenge-em-mollie'),
					'%spaces% &rArr; ' . __('The number of booked tickets', 'stonehenge-em-mollie'),
					'%ticket_name% &rArr; ' . __('The name of the booked ticket', 'stonehenge-em-mollie'),
					__('These wildcards can only be used in Single Bookings Mode:', 'stonehenge-em-mollie'),
					'%event_date% &rArr; ' . __('The start date of the event, using the EM date format', 'stonehenge-em-mollie'),
					'%event_name% &rArr; ' . __('The name of the booked event', 'stonehenge-em-mollie')
				),
			),
			array(
				'id'		=> 'send_cancel_mail',
				'label'  	=> __('Send email on failed / cancelled payment?', 'stonehenge-em-mollie'),
				'type' 		=> 'select',
				'default' 	=> 'yes',
				'help'		=> __('By default Events Manager will send the Booking Cancelled Email if a payment had failed or is incomplete. This can lead to confusion if the user rebooks right after with a successful payment. This option lets you disable sending the automatic Booking Cancelled Email. (Setting this option to "no" will not affect the email if you change the booking status manually.)', 'stonehenge-em-mollie'),
			),
		);
		return $fields_array;
	}


	#===============================================
	# 	Create the Gateway Settings Page.
	#===============================================
	function mysettings() {
		global $EM_options;

		echo '<table class="form-table">';
		foreach($this->define_settings_fields() as $fields => $field) {
			$field['id'] 	= strtolower($field['id']);							// Just in case.
			$field['name'] 	= 'em_mollie_'. $field['id'];
			$field['value'] = !empty(get_option( $field['name'])) ? get_option($field['name']) : $field['default'];
			$helper    		= !empty($field['help']) ? '<p class="description">'.$field['help'].'</p>' : '';

			echo '<tr>';
			echo '<th scope="row">'.$field['label'].'</th>';
			echo '<td>';

			switch($field['type']) {
				case 'page':
					if($field['value']) { $idSelectPage = $field['value']; } else { $idSelectPage = 0; }
					$args = array(
						'name'    			=> $field['name'],
						'selected'     		=> $idSelectPage,
						'show_option_none' 	=> '['. __('None', 'stonehenge-em-mollie') . ']',
					);
					wp_dropdown_pages($args);
				break;
				case 'select':
					echo '<select name="'.$field['name'].'">';
					if(!empty($field['choices'])) {
						foreach($field['choices'] as $v => $l) {
							$selected = $v === $field['value'] ? 'selected="selected"' : '';
							printf('<option %s value="%s">%s</option>', $selected, $v, $l);
						}
					} else {
						$selected = $field['value'];
						?>
						<option value="no" <?php echo ($selected == 'no') ? 'selected="selected"' : ''; ?>><?php _e('No'); ?></option>
						<option value="yes" <?php echo ($selected == 'yes') ? 'selected="selected"' : ''; ?>><?php _e('Yes'); ?></option>
						<?php ;
					}
					echo '</select>';
				break;
				default:
					echo '<input type="text" name="'.$field['name'].'" value="'. __($field['value'], 'stonehenge-em-mollie') .'" class="regular-text" required="required">';
				break;
			}
			echo $helper;
			echo '</td>';
			echo '</tr>';
		}
		echo '</table>';
	}


	#===============================================
	# 	Save or update the Gateway Settings Page options.
	#===============================================
	function update() {
		// Hook into function of Events Manager ->handles sanitation for all inputs.
		parent::update();
		foreach( $this->define_settings_fields() as $fields => $field ) {
			$field['id'] = 'em_'.$this->gateway.'_'. $field['id'];
			update_option( $field['id'], stripslashes($_REQUEST[$field['id']]) );
		}
		return true;
	}

} // End class.

EM_Gateways::register_gateway('mollie', 'EM_Gateway_Mollie');
